package viewModels
{
    import events.CountEvent;

    import flash.events.Event;
    import flash.events.IEventDispatcher;

    /**
     * Default implementation of ICounterViewModel
     */
    public class CounterViewModel extends ViewModelBase
                                  implements ICounterViewModel
    {
        private static const COUNT_CHANGED:String = "countChanged";

        private static const COUNTER_TITLE:String = "Counter ";

        private static var counterNumber:int;

        //------------------------------------
        //
        //           Constructor
        //
        //------------------------------------

        /**
         * Constructor
         */
        public function CounterViewModel(dispatcher:IEventDispatcher)
        {
            super(dispatcher);
            counterNumber++;
        }


        //------------------------------------
        //
        //           Properties
        //
        //------------------------------------

        //----------------------------------
        //  title
        //----------------------------------

        /**
         * @see ICounterViewModel.title
         */
        public function get title():String
        {
            return COUNTER_TITLE + counterNumber;
        }

        //----------------------------------
        //  globalCount
        //----------------------------------

        private var _globalCount:int;
        [Bindable]
        /**
         * @see ICounterViewModel.globalCount
         */
        public function get globalCount():int
        {
            return _globalCount;
        }
        /**
         * @private
         */
        public function set globalCount(value:int):void
        {
            _globalCount = value;
        }

        //----------------------------------
        //  count
        //----------------------------------

        private var _count:int;
        [Bindable("countChanged")]
        /**
         * @see ICounterViewModel.count
         */
        public function get count():int
        {
            return _count;
        }
        /**
         * @private
         */
        public function set count(value:int):void
        {
            if (value != _count)
            {
                _count = value;
                dispatchEvent(new Event(COUNT_CHANGED));
                updateSubCounters();
            }
        }

        //----------------------------------
        //  subCounters
        //----------------------------------

        private var _subCounters:Array;
        /**
         * @see ICounterViewModel.subCounters
         */
        public function get subCounters():Array
        {
            return _subCounters;
        }
        /**
         * @private
         */
        public function set subCounters(value:Array):void
        {
            _subCounters = value;
            updateSubCounters();
        }


        //------------------------------------
        //
        //           Functions
        //
        //------------------------------------

        /**
         * Ensures the local sub-total and global counts are appropriately updated.
         */
        public function updateCounts(newCount:int):void
        {
            var delta:int = newCount - count;
            count = newCount;
            globalCount += delta;

            // inform everyone else that the count has changed
            dispatchEvent(new CountEvent(CountEvent.GLOBAL_UPDATE, globalCount));
        }

        /**
         * Ensures the sub-total is correctly set on all sub-counters.
         */
        private function updateSubCounters():void
        {
            if (_subCounters)
            {
                for each (var subCounter:ISubCounterViewModel in _subCounters)
                {
                    subCounter.subTotal = count;
                }
            }
        }
    }
}