package viewModels
{
    import events.CountEvent;

    import flash.events.Event;
    import flash.events.IEventDispatcher;
    import flash.events.TimerEvent;
    import flash.utils.Timer;

    import views.CounterView;

    /**
     * Default implementation of IMainViewModel
     */
    public class MainViewModel extends ViewModelBase
                               implements IMainViewModel
    {
        private static const TITLE_CHANGED:String = "titleChanged";
        private static const COUNT_CHANGED:String = "countChanged";
        private static const COUNTERS_CHANGED:String = "countersChanged";
        private static const TOGGLE_COUNTDOWN_LABEL_CHANGED:String = "toggleCountdownLabelChanged";

        private static const START_COUNTDOWN:String = "START COUNTDOWN";
        private static const STOP_COUNTDOWN:String = "STOP COUNTDOWN";

        private var _timer:Timer;

        //------------------------------------
        //
        //           Constructor
        //
        //------------------------------------

        /**
         * Constructor
         */
        public function MainViewModel(dispatcher:IEventDispatcher)
        {
            super(dispatcher);
            initialise();
        }


        //------------------------------------
        //
        //           Properties
        //
        //------------------------------------

        //----------------------------------
        //  title
        //----------------------------------

        [Bindable("titleChanged")]
        /**
         * @see IMainViewModel.title
         */
        public function get title():String
        {
            return "MainView (" + countdown + ")";
        }

        //----------------------------------
        //  count
        //----------------------------------

        private var _count:int;
        [Bindable("countChanged")]
        /**
         * @see IMainViewModel.count
         */
        public function get count():int
        {
            return _count;
        }
        /**
         * @private
         */
        public function set count(value:int):void
        {
            // only update if the value has actually changed
            if (value != _count)
            {
                _count = value;
                dispatchEvent(new Event(COUNT_CHANGED));
            }
        }

        //----------------------------------
        //  canDeleteCounter
        //----------------------------------

        [Bindable("countersChanged")]
        /**
         * @see IMainViewModel.canDeleteCounter
         */
        public function get canDeleteCounter():Boolean
        {
            return _counters ? _counters.length > 0 : false;
        }

        //----------------------------------
        //  toggleCountdownLabel
        //----------------------------------

        [Bindable("toggleCountdownLabelChanged")]
        /**
         * @see IMainViewModel.toggleCountdownLabel
         */
        public function get toggleCountdownLabel():String
        {
            return _timer.running ? STOP_COUNTDOWN : START_COUNTDOWN;
        }

        //----------------------------------
        //  counters
        //----------------------------------

        private var _counters:Array = [];
        [Bindable("countersChanged")]
        /**
         * @see IMainViewModel.counters
         */
        public function get counters():Array
        {
            return _counters;
        }

        //----------------------------------
        //  countdown
        //----------------------------------

        private var _countdown:int = 9;
        /**
         * The current countdown value.
         */
        protected function get countdown():int
        {
            return _countdown;
        }
        /**
         * @private
         */
        protected function set countdown(value:int):void
        {
            if (value != _countdown)
            {
                _countdown = value;
                dispatchEvent(new Event(TITLE_CHANGED));
            }
        }


        //------------------------------------
        //
        //           Functions
        //
        //------------------------------------

        /**
         * @see IMainViewModel.increaseCount
         */
        public function increaseCount():void
        {
            count++;

            // inform everyone else that the count has changed
            dispatchEvent(new CountEvent(CountEvent.GLOBAL_UPDATE, count));
        }

        /**
         * @see IMainViewModel.decreaseCount
         */
        public function decreaseCount():void
        {
            count--

            // inform everyone else that the count has changed
            dispatchEvent(new CountEvent(CountEvent.GLOBAL_UPDATE, _count));
        }

        /**
         * @see IMainViewModel.addCounter
         */
        public function addCounter():void
        {
            _counters.push(new CounterView());
            dispatchEvent(new Event(COUNTERS_CHANGED));
        }

        /**
         * @see IMainViewModel.deleteCounter
         */
        public function deleteCounter():void
        {
            _counters.pop();
            dispatchEvent(new Event(COUNTERS_CHANGED));
        }

        /**
         * @see IMainViewModel.toggleCountdown
         */
        public function toggleCountdown():void
        {
            if (_timer.running)
            {
                _timer.stop();
            }
            else
            {
                _timer.start();
            }
            dispatchEvent(new Event(TOGGLE_COUNTDOWN_LABEL_CHANGED));
        }

        /**
         * Initialises the countdown timer used by the class.
         */
        private function initialise():void
        {
            _timer = new Timer(1000, 0);
            _timer.addEventListener(TimerEvent.TIMER, timerEventHandler, false, 0, true);
            _timer.start();
        }


        //------------------------------------
        //
        //           Event Handlers
        //
        //------------------------------------

        /**
         *
         */
        private function timerEventHandler(event:TimerEvent):void
        {
            if (countdown > 0)
            {
                countdown--;
            }
            else
            {
                countdown = 9;
                dispatchEvent(new CountEvent(CountEvent.RESET, 0));
            }
        }
    }
}