package viewModels
{
    import events.CountEvent;

    import flexunit.framework.Assert;

    import mx.collections.IList;

    import org.flexunit.asserts.assertEquals;
    import org.flexunit.asserts.assertNotNull;
    import org.flexunit.asserts.assertTrue;

    import utils.TestEventDispatcher;

    public class CounterViewModelTest
    {
        [Test]
        public function testTitle():void
        {
            var viewModel:CounterViewModel = new CounterViewModel(null);
            assertNotNull("title", viewModel.title);
        }

        [Test]
        public function testGlobalCount():void
        {
            var viewModel:CounterViewModel = new CounterViewModel(null);
            assertEquals("0 initial global count", 0, viewModel.globalCount);

            var expected:int = 7;
            viewModel.globalCount = expected;
            assertEquals("global count", expected, viewModel.globalCount);
        }

        [Test]
        public function testSubCounters():void
        {
            var subTotal:int = 8;
            var viewModel:CounterViewModel = new CounterViewModel(null);
            viewModel.count = subTotal;

            var subCounters:Array = [ new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel() ];
            var subCounter:MockSubCounterViewModel;
            for each (subCounter in subCounters)
            {
                subCounter.expects("set_subTotal").withArg(subTotal);
            }
            viewModel.subCounters = subCounters;
            for each (subCounter in subCounters)
            {
                assertTrue(subCounter.errorMessage(), subCounter.success());
            }
        }

        [Test]
        public function testCount():void
        {
            // setup
            var viewModel:CounterViewModel = new CounterViewModel(null);
            var subCounters:Array = [ new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel() ];
            for each (subCounter in subCounters)
            {
                subCounter.expects("set_subTotal").withArg(viewModel.count);
            }
            viewModel.subCounters = subCounters;

            // test
            var expected:int = 42;
            var subCounter:MockSubCounterViewModel;
            for each (subCounter in subCounters)
            {
                subCounter.expects("set_subTotal").withArg(expected);
            }
            viewModel.count = expected;
            for each (subCounter in subCounters)
            {
                assertTrue(subCounter.errorMessage(), subCounter.success());
            }
        }

        [Test]
        public function testUpdateCounts():void
        {
            // setup
            var dispatcher:TestEventDispatcher = new TestEventDispatcher();
            var viewModel:CounterViewModel = new CounterViewModel(dispatcher);
            var globalCount:int = 8;
            var subTotal:int = 4;
            viewModel.count = subTotal;
            viewModel.globalCount = globalCount;
            var subCounters:Array = [ new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel(),
                                      new MockSubCounterViewModel() ];
            var subCounter:MockSubCounterViewModel;
            for each (subCounter in subCounters)
            {
                subCounter.expects("set_subTotal").withArg(viewModel.count);
            }
            viewModel.subCounters = subCounters;

            // test
            var expectedSubTotal:int = 6;
            var expectedGlobal:int = globalCount + (expectedSubTotal - subTotal);
            for each (subCounter in subCounters)
            {
                subCounter.expects("set_subTotal").withArg(expectedSubTotal);
            }
            viewModel.updateCounts(expectedSubTotal);
            assertEquals("new global count", expectedGlobal, viewModel.globalCount);
            assertEquals("new sub-total", expectedSubTotal, viewModel.count);
            for each (subCounter in subCounters)
            {
                assertTrue(subCounter.errorMessage(), subCounter.success());
            }
            var dispatchedEvents:IList = dispatcher.eventsForType(CountEvent.GLOBAL_UPDATE);
            assertEquals("1 global update event", 1, dispatchedEvents.length);
            assertEquals("event indicates global", expectedGlobal, CountEvent(dispatchedEvents[0]).newCount);
        }
    }
}

import flash.events.Event;

import org.mock4as.Mock;

import viewModels.ISubCounterViewModel;

class MockSubCounterViewModel extends Mock
                              implements ISubCounterViewModel
{
    public function get subTotal():int
    {
        record("get_subTotal");
        return int(expectedReturnFor("get_subTotal"));
    }
    public function set subTotal(value:int):void
    {
        record("set_subTotal", value);
    }

    public function get count():int
    {
        record("count");
        return int(expectedReturnFor("count"));
    }

    public function increaseCount():void
    {
        record("increaseCount");
    }

    public function decreaseCount():void
    {
        record("decreaseCount");
    }

    public function addEventListener(type:String,
                                     listener:Function,
                                     useCapture:Boolean = false,
                                     priority:int = 0,
                                     useWeakReference:Boolean = false):void
    {
        record("addEventListener");
    }

    public function removeEventListener(type:String,
                                        listener:Function,
                                        useCapture:Boolean = false):void
    {
        record("removeEventListener");
    }

    public function dispatchEvent(event:Event):Boolean
    {
        record("dispatchEvent");
        return expectedReturnFor("dispatchEvent");
    }

    public function hasEventListener(type:String):Boolean
    {
        record("hasEventListener");
        return expectedReturnFor("hasEventListener");
    }

    public function willTrigger(type:String):Boolean
    {
        record("willTrigger");
        return expectedReturnFor("willTrigger");
    }
}